import tape from 'tape';
import {
  is,
  fromJS,
  Iterable,
  List,
  Map,
  OrderedMap
} from 'immutable';

import {
  deepReduce,
  deepReduceLeaves,
  deepReduceParents,
  deepReduceOutwards,
  deepReduceLeavesOutwards,
  deepReduceParentsOutwards
} from '../reduce';

import {
  isLeaf
} from '../utils';

const nestedMaps = fromJS({
  a: {
    b: "B",
    c: "C",
    d: {
      e: "E"
    }
  },
  f: "F"
});

const childLists = fromJS({
  name: "root",
  children: [
    {
      name: "child 1",
      children: [
        {name: "grandchild 1"},
        {name: "grandchild 2"}
      ]
    },
    {
      name: "child 2",
      children: []
    },
    {name: "child 3"}
  ]
});

tape('deepReduce', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduce((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"grandchild 1",
    		"grandchild 2",
    		"child 1",
    		"child 2",
    		"child 3",
    		"root"
    	])
    ), 'deepReduce iterates through every node using a childPath, from the outside in');

    test.ok(is(
    	nestedMaps.update(deepReduce((reduction, item) => {
    		// normally deepReduce on trees that don't require childPaths is a bit pointless
    		// because you'll almost certainly have to deal with leaves and parents differently
    		const a = isLeaf(item) ? item : "...";
    		return reduction.push(a);
    	}, List())),
    	fromJS([
    		"B",
    		"C",
    		"E",
    		"...",
    		"...",
    		"F",
    		"..."
    	])
    ), 'deepReduce iterates through every node without using a childPath (kind of pointless)');
});

tape('deepReduceLeaves', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduceLeaves((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"grandchild 1",
    		"grandchild 2",
    		"child 2",
    		"child 3"
    	])
    ), 'deepReduceLeaves iterates through every leaf node using a childPath, from the outside in');

    test.ok(is(
    	nestedMaps.update(deepReduceLeaves((reduction, item) => {
    		return reduction.push(item);
    	}, List())),
    	fromJS([
    		"B",
    		"C",
    		"E",
    		"F"
    	])
    ), 'deepReduceLeaves iterates through every leaf node without using a childPath');
});


tape('deepReduceParents', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduceParents((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"child 1",
    		"root"
    	])
    ), 'deepReduceParents iterates through every parent node using a childPath, from the outside in');

    test.ok(is(
    	nestedMaps.update(deepReduceParents((reduction, item) => {
    		return reduction.push(item.size);
    	}, List())),
    	fromJS([
    		1,
    		3,
    		2
    	])
    ), 'deepReduceParents iterates through every parent node without using a childPath');
});

tape('deepReduceOutwards', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduceOutwards((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"root",
    		"child 1",
    		"grandchild 1",
    		"grandchild 2",
    		"child 2",
    		"child 3"
    	])
    ), 'deepReduceOutwards iterates through every node using a childPath, from the inside out');

    test.ok(is(
    	nestedMaps.update(deepReduceOutwards((reduction, item) => {
    		// normally deepReduceOutwards on trees that don't require childPaths is a bit pointless
    		// because you'll almost certainly have to deal with leaves and parents differently
    		const a = isLeaf(item) ? item : "...";
    		return reduction.push(a);
    	}, List())),
    	fromJS([
    		"...",
    		"...",
    		"B",
    		"C",
    		"...",
    		"E",
    		"F"
    	])
    ), 'deepReduceOutwards iterates through every node without using a childPath, from the inside out');
});

tape('deepReduceLeavesOutwards', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduceLeavesOutwards((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"grandchild 1",
    		"grandchild 2",
    		"child 2",
    		"child 3"
    	])
    ), 'deepReduceLeavesOutwards iterates through every leaf node using a childPath, from the inside out');

    test.ok(is(
    	nestedMaps.update(deepReduceLeavesOutwards((reduction, item) => {
    		return reduction.push(item);
    	}, List())),
    	fromJS([
    		"B",
    		"C",
    		"E",
    		"F"
    	])
    ), 'deepReduceLeavesOutwards iterates through every leaf node without using a childPath, from the inside out');
});

tape('deepReduceParentsOutwards', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepReduceParentsOutwards((reduction, item) => reduction.push(item.get('name')), List(), ['children'])),
    	fromJS([
    		"root",
    		"child 1"
    	])
    ), 'deepReduceParentsOutwards iterates through every parent node using a childPath, from the inside out');

    test.ok(is(
    	nestedMaps.update(deepReduceParentsOutwards((reduction, item) => {
    		return reduction.push(item.size);
    	}, List())),
    	fromJS([
    		2,
    		3,
    		1
    	])
    ), 'deepReduceParentsOutwards iterates through every parent node without using a childPath, from the inside out');
});